@@ -16,6 +16,7 @@ package operator
 
 import (
 	"context"
+	"fmt"
 	"strings"
 	"time"
 
@@ -23,6 +24,7 @@ import (
 	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	v1 "k8s.io/api/core/v1"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/client-go/rest"
@@ -51,18 +53,16 @@ const (
 type Operator struct {
 	namespace, namespaceUserWorkload string
 
-	configMapName    string
-	images           map[string]string
-	telemetryMatches []string
-	remoteWrite      bool
+	configMapName             string
+	userWorkloadConfigMapName string
+	images                    map[string]string
+	telemetryMatches          []string
+	remoteWrite               bool
 
 	client *client.Client
 
-	cmapInf                       cache.SharedIndexInformer
-	kubeSystemCmapInf             cache.SharedIndexInformer
-	openshiftConfigManagedCmapInf cache.SharedIndexInformer
-	openshiftConfigCmapInf        cache.SharedIndexInformer
-	secretInf                     cache.SharedIndexInformer
+	cmapInf   cache.SharedIndexInformer
+	informers []cache.SharedIndexInformer
 
 	queue workqueue.RateLimitingInterface
 
@@ -70,34 +70,37 @@ type Operator struct {
 	reconcileErrors   prometheus.Counter
 }
 
-func New(config *rest.Config, version, namespace, namespaceUserWorkload, namespaceSelector, configMapName string, remoteWrite bool, images map[string]string, telemetryMatches []string) (*Operator, error) {
-	c, err := client.New(config, version, namespace, namespaceSelector)
+func New(config *rest.Config, version, namespace, namespaceUserWorkload, namespaceSelector, configMapName, userWorkloadConfigMapName string, remoteWrite bool, images map[string]string, telemetryMatches []string) (*Operator, error) {
+	c, err := client.New(config, version, namespace, namespaceUserWorkload, namespaceSelector)
 	if err != nil {
 		return nil, err
 	}
 
 	o := &Operator{
-		images:                images,
-		telemetryMatches:      telemetryMatches,
-		configMapName:         configMapName,
-		remoteWrite:           remoteWrite,
-		namespace:             namespace,
-		namespaceUserWorkload: namespaceUserWorkload,
-		client:                c,
-		queue:                 workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "cluster-monitoring"),
+		images:                    images,
+		telemetryMatches:          telemetryMatches,
+		configMapName:             configMapName,
+		userWorkloadConfigMapName: userWorkloadConfigMapName,
+		remoteWrite:               remoteWrite,
+		namespace:                 namespace,
+		namespaceUserWorkload:     namespaceUserWorkload,
+		client:                    c,
+		queue:                     workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "cluster-monitoring"),
+		informers:                 make([]cache.SharedIndexInformer, 0),
 	}
 
-	o.secretInf = cache.NewSharedIndexInformer(
+	informer := cache.NewSharedIndexInformer(
 		o.client.SecretListWatchForNamespace(namespace), &v1.Secret{}, resyncPeriod, cache.Indexers{},
 	)
-	o.secretInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
+	informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
 		AddFunc:    o.handleEvent,
 		UpdateFunc: func(_, newObj interface{}) { o.handleEvent(newObj) },
 		DeleteFunc: o.handleEvent,
 	})
+	o.informers = append(o.informers, informer)
 
 	o.cmapInf = cache.NewSharedIndexInformer(
-		o.client.ConfigMapListWatch(), &v1.ConfigMap{}, resyncPeriod, cache.Indexers{},
+		o.client.ConfigMapListWatchForNamespace(namespace), &v1.ConfigMap{}, resyncPeriod, cache.Indexers{},
 	)
 	o.cmapInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
 		AddFunc:    o.handleEvent,
@@ -105,29 +108,42 @@ func New(config *rest.Config, version, namespace, namespaceUserWorkload, namespa
 		DeleteFunc: o.handleEvent,
 	})
 
-	o.kubeSystemCmapInf = cache.NewSharedIndexInformer(
+	informer = cache.NewSharedIndexInformer(
+		o.client.ConfigMapListWatchForNamespace(namespaceUserWorkload), &v1.ConfigMap{}, resyncPeriod, cache.Indexers{},
+	)
+	informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
+		AddFunc:    o.handleEvent,
+		UpdateFunc: func(_, newObj interface{}) { o.handleEvent(newObj) },
+		DeleteFunc: o.handleEvent,
+	})
+	o.informers = append(o.informers, informer)
+
+	informer = cache.NewSharedIndexInformer(
 		o.client.ConfigMapListWatchForNamespace("kube-system"),
 		&v1.ConfigMap{}, resyncPeriod, cache.Indexers{},
 	)
-	o.kubeSystemCmapInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
+	informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
 		UpdateFunc: func(_, newObj interface{}) { o.handleEvent(newObj) },
 	})
+	o.informers = append(o.informers, informer)
 
-	o.openshiftConfigManagedCmapInf = cache.NewSharedIndexInformer(
+	informer = cache.NewSharedIndexInformer(
 		o.client.ConfigMapListWatchForNamespace("openshift-config-managed"),
 		&v1.ConfigMap{}, resyncPeriod, cache.Indexers{},
 	)
-	o.openshiftConfigManagedCmapInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
+	informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
 		UpdateFunc: func(_, newObj interface{}) { o.handleEvent(newObj) },
 	})
+	o.informers = append(o.informers, informer)
 
-	o.openshiftConfigCmapInf = cache.NewSharedIndexInformer(
+	informer = cache.NewSharedIndexInformer(
 		o.client.ConfigMapListWatchForNamespace("openshift-config"),
 		&v1.ConfigMap{}, resyncPeriod, cache.Indexers{},
 	)
-	o.openshiftConfigCmapInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
+	informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
 		UpdateFunc: func(_, newObj interface{}) { o.handleEvent(newObj) },
 	})
+	o.informers = append(o.informers, informer)
 
 	return o, nil
 }
@@ -176,11 +192,14 @@ func (o *Operator) Run(stopc <-chan struct{}) error {
 	}
 
 	go o.cmapInf.Run(stopc)
-	go o.secretInf.Run(stopc)
-	go o.kubeSystemCmapInf.Run(stopc)
+	synced := []cache.InformerSynced{o.cmapInf.HasSynced}
+	for _, inf := range o.informers {
+		go inf.Run(stopc)
+		synced = append(synced, inf.HasSynced)
+	}
 
 	klog.V(4).Info("Waiting for initial cache sync.")
-	ok := cache.WaitForCacheSync(stopc, o.cmapInf.HasSynced, o.kubeSystemCmapInf.HasSynced)
+	ok := cache.WaitForCacheSync(stopc, synced...)
 	if !ok {
 		return errors.New("failed to sync informers")
 	}
@@ -230,6 +249,7 @@ func (o *Operator) handleEvent(obj interface{}) {
 	klog.V(5).Infof("ConfigMap or Secret updated: %s", key)
 
 	cmoConfigMap := o.namespace + "/" + o.configMapName
+	uwmConfigMap := o.namespaceUserWorkload + "/" + o.userWorkloadConfigMapName
 
 	switch key {
 	case cmoConfigMap:
@@ -240,11 +260,14 @@ func (o *Operator) handleEvent(obj interface{}) {
 	case telemeterCABundleConfigMap:
 	case alertmanagerCABundleConfigMap:
 	case grpcTLS:
+	case uwmConfigMap:
 	default:
 		klog.V(5).Infof("ConfigMap or Secret (%s) not triggering an update.", key)
 		return
 	}
 
+	klog.Infof("Triggering an update due to ConfigMap or Secret: %s", key)
+
 	// Always enqueue the cluster monitoring operator configmap.
 	// That way we reuse the same synchronization logic for all triggering object changes.
 	o.enqueue(cmoConfigMap)
@@ -313,11 +336,11 @@ func (o *Operator) sync(key string) error {
 		o.client,
 		[]*tasks.TaskSpec{
 			tasks.NewTaskSpec("Updating Prometheus Operator", tasks.NewPrometheusOperatorTask(o.client, factory)),
-			tasks.NewTaskSpec("Updating user workload Prometheus Operator", tasks.NewPrometheusOperatorUserWorkloadTask(o.client, factory, config.UserWorkloadConfig)),
+			tasks.NewTaskSpec("Updating user workload Prometheus Operator", tasks.NewPrometheusOperatorUserWorkloadTask(o.client, factory, config)),
 			tasks.NewTaskSpec("Updating Cluster Monitoring Operator", tasks.NewClusterMonitoringOperatorTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating Grafana", tasks.NewGrafanaTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating Prometheus-k8s", tasks.NewPrometheusTask(o.client, factory, config)),
-			tasks.NewTaskSpec("Updating Prometheus-user-workload", tasks.NewPrometheusUserWorkloadTask(o.client, factory, config.UserWorkloadConfig)),
+			tasks.NewTaskSpec("Updating Prometheus-user-workload", tasks.NewPrometheusUserWorkloadTask(o.client, factory, config)),
 			tasks.NewTaskSpec("Updating Alertmanager", tasks.NewAlertmanagerTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating node-exporter", tasks.NewNodeExporterTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating kube-state-metrics", tasks.NewKubeStateMetricsTask(o.client, factory)),
@@ -325,8 +348,8 @@ func (o *Operator) sync(key string) error {
 			tasks.NewTaskSpec("Updating prometheus-adapter", tasks.NewPrometheusAdapterTaks(o.namespace, o.client, factory)),
 			tasks.NewTaskSpec("Updating Telemeter client", tasks.NewTelemeterClientTask(o.client, factory, config)),
 			tasks.NewTaskSpec("Updating configuration sharing", tasks.NewConfigSharingTask(o.client, factory)),
-			tasks.NewTaskSpec("Updating Thanos Querier", tasks.NewThanosQuerierTask(o.client, factory, config.UserWorkloadConfig)),
-			tasks.NewTaskSpec("Updating User Workload Thanos Ruler", tasks.NewThanosRulerUserWorkloadTask(o.client, factory, config.UserWorkloadConfig)),
+			tasks.NewTaskSpec("Updating Thanos Querier", tasks.NewThanosQuerierTask(o.client, factory, config)),
+			tasks.NewTaskSpec("Updating User Workload Thanos Ruler", tasks.NewThanosRulerUserWorkloadTask(o.client, factory, config)),
 		},
 	)
 
@@ -356,6 +379,34 @@ func (o *Operator) sync(key string) error {
 	return nil
 }
 
+func (o *Operator) loadUserWorkloadConfig() (*manifests.UserWorkloadConfiguration, error) {
+	cmKey := fmt.Sprintf("%s/%s", o.namespaceUserWorkload, o.userWorkloadConfigMapName)
+
+	userCM, err := o.client.GetConfigmap(o.namespaceUserWorkload, o.userWorkloadConfigMapName)
+	if err != nil {
+		if apierrors.IsNotFound(err) {
+			klog.Warningf("User Workload Monitoring %q ConfigMap not found. Using defaults.", cmKey)
+			return manifests.NewDefaultUserWorkloadMonitoringConfig(), nil
+		}
+		klog.Warningf("Error loading User Workload Monitoring %q ConfigMap. Error: %v", cmKey, err)
+		return nil, errors.Wrapf(err, "the User Workload Monitoring %q ConfigMap could not be loaded", cmKey)
+	}
+
+	const configKey = "config.yaml"
+	configContent, found := userCM.Data[configKey]
+	if !found {
+		klog.Warningf("No %q key found in User Workload Monitoring %q ConfigMap. Using defaults.", configKey, cmKey)
+		return manifests.NewDefaultUserWorkloadMonitoringConfig(), nil
+	}
+
+	uwc, err := manifests.NewUserConfigFromString(configContent)
+	if err != nil {
+		klog.Warningf("Error creating User Workload Configuration from %q key in the %q ConfigMap. Error: %v", configKey, cmKey, err)
+		return nil, errors.Wrapf(err, "the User Workload Configuration from %q key in the %q ConfigMap could not be parsed", configKey, cmKey)
+	}
+	return uwc, nil
+}
+
 func (o *Operator) loadConfig(key string) (*manifests.Config, error) {
 	obj, found, err := o.cmapInf.GetStore().GetByKey(key)
 	if err != nil {
@@ -388,8 +439,21 @@ func (o *Operator) Config(key string) (*manifests.Config, error) {
 		return nil, err
 	}
 
+	// Only use User Workload Monitoring ConfigMap from user ns and populate if
+	// its enabled by admin via Cluster Monitoring ConfigMap.  The above
+	// loadConfig() already initializes the structs with nil values for
+	// UserWorkloadConfiguration struct.
+	if *c.ClusterMonitoringConfiguration.UserWorkloadEnabled {
+		c.UserWorkloadConfiguration, err = o.loadUserWorkloadConfig()
+		if err != nil {
+			return nil, err
+		}
+	} else if c.ClusterMonitoringConfiguration.UserWorkloadConfig != nil && c.ClusterMonitoringConfiguration.UserWorkloadConfig.Enabled != nil && *c.ClusterMonitoringConfiguration.UserWorkloadConfig.Enabled {
+		klog.Warningf("User Workload Monitoring enabled via the deprecated 'techPreviewUserWorkload' setting in %q configmap. Use the 'enableUserWorkload' setting instead.", key)
+	}
+
 	// Only fetch the token and cluster ID if they have not been specified in the config.
-	if c.TelemeterClientConfig.ClusterID == "" || c.TelemeterClientConfig.Token == "" {
+	if c.ClusterMonitoringConfiguration.TelemeterClientConfig.ClusterID == "" || c.ClusterMonitoringConfiguration.TelemeterClientConfig.Token == "" {
 		err := c.LoadClusterID(func() (*configv1.ClusterVersion, error) {
 			return o.client.GetClusterVersion("version")
 		})
@@ -442,7 +506,7 @@ func (o *Operator) Config(key string) (*manifests.Config, error) {
 		keyFound && len(keyContent) > 0 {
 
 		trueBool := true
-		c.EtcdConfig.Enabled = &trueBool
+		c.ClusterMonitoringConfiguration.EtcdConfig.Enabled = &trueBool
 	}
 
 	return c, nil
